# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.initial_term import InitialTerm
from zuora_sdk.models.renewal_term import RenewalTerm
from typing import Optional, Set
from typing_extensions import Self

class OrderActionTermsAndConditions(BaseModel):
    """
    Information about an order action of type `TermsAndConditions`. 
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    auto_renew: Optional[StrictBool] = Field(default=None, alias="autoRenew")
    bill_to_contact_id: Optional[StrictStr] = Field(default=None, description="The ID of the bill-to contact associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Contact from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ", alias="billToContactId")
    clearing_existing_bill_to_contact: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing bill-to contact ID at the subscription level. This field is mutually exclusive with the `billToContactId` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingBillToContact")
    clearing_existing_invoice_group_number: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing invoice group number at the subscription level. This field is mutually exclusive with the `invoiceGroupNumber` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingInvoiceGroupNumber")
    clearing_existing_invoice_template: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing invoice template ID at the subscription level. This field is mutually exclusive with the `invoiceTemplateId` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingInvoiceTemplate")
    clearing_existing_payment_term: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing payment term at the subscription level. This field is mutually exclusive with the `paymentTerm` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingPaymentTerm")
    clearing_existing_sequence_set: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing sequence set ID at the subscription level. This field is mutually exclusive with the `sequenceSetId` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingSequenceSet")
    clearing_existing_sold_to_contact: Optional[StrictBool] = Field(default=False, description="Whether to clear the existing sold-to contact ID at the subscription level. This field is mutually exclusive with the `soldToContactId` field.  **Note**: If you have the [Flexible Billing Attributes](https://knowledgecenter.zuora.com/Billing/Subscriptions/Flexible_Billing_Attributes) feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body. ", alias="clearingExistingSoldToContact")
    initial_term: Optional[InitialTerm] = Field(default=None, alias="initialTerm")
    invoice_separately: Optional[StrictBool] = Field(default=None, description="Specifies whether the subscription appears on a separate invoice when Zuora generates invoices.", alias="invoiceSeparately")
    invoice_group_number: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The number of invoice group associated with the subscription.  **Note**: This field is available only if you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature enabled. ", alias="invoiceGroupNumber")
    invoice_template_id: Optional[StrictStr] = Field(default=None, description="The ID of the invoice template associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Template from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ", alias="invoiceTemplateId")
    payment_term: Optional[StrictStr] = Field(default=None, description="The name of the payment term associated with the subscription. For example, `Net 30`. The payment term determines the due dates of invoices.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Term from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ", alias="paymentTerm")
    renewal_setting: Optional[StrictStr] = Field(default=None, alias="renewalSetting")
    renewal_terms: Optional[List[RenewalTerm]] = Field(default=None, alias="renewalTerms")
    sequence_set_id: Optional[StrictStr] = Field(default=None, description="The ID of the sequence set associated with the subscription.    **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Set from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ", alias="sequenceSetId")
    sold_to_contact_id: Optional[StrictStr] = Field(default=None, description="The ID of the sold-to contact associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Contact from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ", alias="soldToContactId")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["autoRenew", "billToContactId", "clearingExistingBillToContact", "clearingExistingInvoiceGroupNumber", "clearingExistingInvoiceTemplate", "clearingExistingPaymentTerm", "clearingExistingSequenceSet", "clearingExistingSoldToContact", "initialTerm", "invoiceSeparately", "invoiceGroupNumber", "invoiceTemplateId", "paymentTerm", "renewalSetting", "renewalTerms", "sequenceSetId", "soldToContactId"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OrderActionTermsAndConditions from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of initial_term
        if self.initial_term:
            _dict['initialTerm'] = self.initial_term.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in renewal_terms (list)
        _items = []
        if self.renewal_terms:
            for _item_renewal_terms in self.renewal_terms:
                if _item_renewal_terms:
                    _items.append(_item_renewal_terms.to_dict())
            _dict['renewalTerms'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OrderActionTermsAndConditions from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "autoRenew": obj.get("autoRenew"),
            "billToContactId": obj.get("billToContactId"),
            "clearingExistingBillToContact": obj.get("clearingExistingBillToContact") if obj.get("clearingExistingBillToContact") is not None else False,
            "clearingExistingInvoiceGroupNumber": obj.get("clearingExistingInvoiceGroupNumber") if obj.get("clearingExistingInvoiceGroupNumber") is not None else False,
            "clearingExistingInvoiceTemplate": obj.get("clearingExistingInvoiceTemplate") if obj.get("clearingExistingInvoiceTemplate") is not None else False,
            "clearingExistingPaymentTerm": obj.get("clearingExistingPaymentTerm") if obj.get("clearingExistingPaymentTerm") is not None else False,
            "clearingExistingSequenceSet": obj.get("clearingExistingSequenceSet") if obj.get("clearingExistingSequenceSet") is not None else False,
            "clearingExistingSoldToContact": obj.get("clearingExistingSoldToContact") if obj.get("clearingExistingSoldToContact") is not None else False,
            "initialTerm": InitialTerm.from_dict(obj["initialTerm"]) if obj.get("initialTerm") is not None else None,
            "invoiceSeparately": obj.get("invoiceSeparately"),
            "invoiceGroupNumber": obj.get("invoiceGroupNumber"),
            "invoiceTemplateId": obj.get("invoiceTemplateId"),
            "paymentTerm": obj.get("paymentTerm"),
            "renewalSetting": obj.get("renewalSetting"),
            "renewalTerms": [RenewalTerm.from_dict(_item) for _item in obj["renewalTerms"]] if obj.get("renewalTerms") is not None else None,
            "sequenceSetId": obj.get("sequenceSetId"),
            "soldToContactId": obj.get("soldToContactId")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


def convert_snake_dict_to_camel(_dict: dict, properties: list[str]):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        new_key = snake_to_camel(k)
        if properties is not None and isinstance(properties, list) and new_key in properties:
            new_dict[new_key] = v
        else:
            new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
