# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from zuora_sdk.models.rev_rec_trigger import RevRecTrigger
from zuora_sdk.models.tax_mode import TaxMode
from zuora_sdk.models.update_discount_invoice_item import UpdateDiscountInvoiceItem
from typing import Optional, Set
from typing_extensions import Self

class UpdateInvoiceItem(BaseModel):
    """
    UpdateInvoiceItem
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    integration_id__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="ID of the corresponding object in NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="IntegrationId__NS")
    integration_status__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="Status of the invoice item's synchronization with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="IntegrationStatus__NS")
    sync_date__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="Date when the invoice item was synchronized with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="SyncDate__NS")
    accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code associated with the invoice item.", alias="accountingCode")
    adjustment_liability_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for adjustment liability.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="adjustmentLiabilityAccountingCode")
    adjustment_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for adjustment revenue.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="adjustmentRevenueAccountingCode")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of the invoice item.   - For tax-inclusive invoice items, the amount indicates the invoice item amount including tax.  - For tax-exclusive invoice items, the amount indicates the invoice item amount excluding tax. ")
    charge_date: Optional[StrictStr] = Field(default=None, description="The date when the invoice item is charged, in `yyyy-mm-dd hh:mm:ss` format.", alias="chargeDate")
    charge_name: Optional[StrictStr] = Field(default=None, description="The name of the charge associated with the invoice item.   This field is required if the `productRatePlanChargeId` field is not specified in the request. ", alias="chargeName")
    contract_asset_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for contract asset.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="contractAssetAccountingCode")
    contract_liability_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for contract liability.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="contractLiabilityAccountingCode")
    contract_recognized_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for contract recognized revenue.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="contractRecognizedRevenueAccountingCode")
    deferred_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the deferred revenue, such as Monthly Recurring Liability.  **Note:** This field is only available if you have Zuora Finance enabled.", alias="deferredRevenueAccountingCode")
    description: Optional[StrictStr] = Field(default=None, description="The description of the invoice item.")
    discount_items: Optional[List[UpdateDiscountInvoiceItem]] = Field(default=None, description="Container for discount items. The maximum number of discount items is 10.", alias="discountItems")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude the invoice item from revenue accounting.  **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="excludeItemBillingFromRevenueAccounting")
    id: Optional[StrictStr] = Field(default=None, description="The unique ID of the invoice item.")
    item_type: Optional[StrictStr] = Field(default=None, description="The type of the invoice item.", alias="itemType")
    purchase_order_number: Optional[StrictStr] = Field(default=None, description="The purchase order number associated the invoice item.", alias="purchaseOrderNumber")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of units for the invoice item.")
    recognized_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the recognized revenue, such as Monthly Recurring Charges or Overage Charges. **Note:** This field is only available if you have Zuora Finance enabled.", alias="recognizedRevenueAccountingCode")
    rev_rec_code: Optional[StrictStr] = Field(default=None, description="The revenue recognition code.", alias="revRecCode")
    rev_rec_trigger_condition: Optional[StrictStr] = Field(default=None, alias="revRecTriggerCondition")
    revenue_recognition_rule_name: Optional[StrictStr] = Field(default=None, description="The name of the revenue recognition rule governing the revenueschedule. **Note:** This field is only available if you have Zuora Finance enabled.", alias="revenueRecognitionRuleName")
    service_end_date: Optional[date] = Field(default=None, description="The service end date of the invoice item.", alias="serviceEndDate")
    service_start_date: Optional[date] = Field(default=None, description="The service start date of the invoice item.", alias="serviceStartDate")
    sku: Optional[StrictStr] = Field(default=None, description="The SKU of the invoice item. The SKU of the invoice item must be different from the SKU of any existing product.")
    tax_code: Optional[StrictStr] = Field(default=None, description="The tax code identifies which tax rules and tax rates to apply to the invoice item.  **Note:**  - This field is only available if you have Taxation enabled. - If the values of both `taxCode` and `taxMode` fields are changed to `null` when updating a standalone invoice, the corresponding `invoiceItems` > `taxItems` field and its nested fields specified in the creation request will be removed. ", alias="taxCode")
    tax_mode: Optional[StrictStr] = Field(default=None, alias="taxMode")
    unbilled_receivables_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for unbilled receivables.         **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="unbilledReceivablesAccountingCode")
    unit_price: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The per-unit price of the invoice item.", alias="unitPrice")
    uom: Optional[StrictStr] = Field(default=None, description="The unit of measure.")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["IntegrationId__NS", "IntegrationStatus__NS", "SyncDate__NS", "accountingCode", "adjustmentLiabilityAccountingCode", "adjustmentRevenueAccountingCode", "amount", "chargeDate", "chargeName", "contractAssetAccountingCode", "contractLiabilityAccountingCode", "contractRecognizedRevenueAccountingCode", "deferredRevenueAccountingCode", "description", "discountItems", "excludeItemBillingFromRevenueAccounting", "id", "itemType", "purchaseOrderNumber", "quantity", "recognizedRevenueAccountingCode", "revRecCode", "revRecTriggerCondition", "revenueRecognitionRuleName", "serviceEndDate", "serviceStartDate", "sku", "taxCode", "taxMode", "unbilledReceivablesAccountingCode", "unitPrice", "uom"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateInvoiceItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in discount_items (list)
        _items = []
        if self.discount_items:
            for _item_discount_items in self.discount_items:
                if _item_discount_items:
                    _items.append(_item_discount_items.to_dict())
            _dict['discountItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateInvoiceItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "IntegrationId__NS": obj.get("IntegrationId__NS"),
            "IntegrationStatus__NS": obj.get("IntegrationStatus__NS"),
            "SyncDate__NS": obj.get("SyncDate__NS"),
            "accountingCode": obj.get("accountingCode"),
            "adjustmentLiabilityAccountingCode": obj.get("adjustmentLiabilityAccountingCode"),
            "adjustmentRevenueAccountingCode": obj.get("adjustmentRevenueAccountingCode"),
            "amount": obj.get("amount"),
            "chargeDate": obj.get("chargeDate"),
            "chargeName": obj.get("chargeName"),
            "contractAssetAccountingCode": obj.get("contractAssetAccountingCode"),
            "contractLiabilityAccountingCode": obj.get("contractLiabilityAccountingCode"),
            "contractRecognizedRevenueAccountingCode": obj.get("contractRecognizedRevenueAccountingCode"),
            "deferredRevenueAccountingCode": obj.get("deferredRevenueAccountingCode"),
            "description": obj.get("description"),
            "discountItems": [UpdateDiscountInvoiceItem.from_dict(_item) for _item in obj["discountItems"]] if obj.get("discountItems") is not None else None,
            "excludeItemBillingFromRevenueAccounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "id": obj.get("id"),
            "itemType": obj.get("itemType"),
            "purchaseOrderNumber": obj.get("purchaseOrderNumber"),
            "quantity": obj.get("quantity"),
            "recognizedRevenueAccountingCode": obj.get("recognizedRevenueAccountingCode"),
            "revRecCode": obj.get("revRecCode"),
            "revRecTriggerCondition": obj.get("revRecTriggerCondition"),
            "revenueRecognitionRuleName": obj.get("revenueRecognitionRuleName"),
            "serviceEndDate": obj.get("serviceEndDate"),
            "serviceStartDate": obj.get("serviceStartDate"),
            "sku": obj.get("sku"),
            "taxCode": obj.get("taxCode"),
            "taxMode": obj.get("taxMode"),
            "unbilledReceivablesAccountingCode": obj.get("unbilledReceivablesAccountingCode"),
            "unitPrice": obj.get("unitPrice"),
            "uom": obj.get("uom")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


def convert_snake_dict_to_camel(_dict: dict, properties: list[str]):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        new_key = snake_to_camel(k)
        if properties is not None and isinstance(properties, list) and new_key in properties:
            new_dict[new_key] = v
        else:
            new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
